#ifndef float_image_mscale_H
#define float_image_mscale_H

/* Tools for multiscale image processing. */
/* Last edited on 2009-02-24 00:20:56 by stolfi */

#include <bool.h>
#include <r2.h>
#include <float_image.h>

float_image_t *float_image_mscale_shrink(float_image_t *A, float_image_t *M, int nw);
  /* Returns a version {R} of {A} downscaled by a factor of 1/2. 
  
    The width of {R} is half the width of {A}, rounded up.
    Ditto fot the height.  The number of channels is the same.
  
    Each pixel of {R} is a weighted average of pixels of {A}, within a
    window with {nw  nw} pixels, using binomial weights within
    that window. If {M} is not null, it must be a single-channel image
    with the same size as {A}; in that case, the window weight of each
    pixel of {A} is multiplied by the corresponding sample of {M}
    before averaging.
    
    Samples of {A} which are NAN or lie outside {A}'s domain are
    assumed to have zero weight. If all contributing samples for a
    sample of {R} have zero weight, the sample is set to NAN.
    
    A point {(x,y)} in the domain of {R} corresponds to point
    {(2*x,2*y)} of {A} if {nw} is even, and to {(2*x-0.5,2*y-0.5)} if
    {nw} is odd. */

float_image_t *float_image_mscale_mask_shrink(float_image_t *M, int nw, bool_t harm);
  /* Similar to {float_image_mscale_shrink}, but specialized for weight masks.
   
    Each pixel of the resulting image {R} is the average of pixels of
    {M}, within a window with {nw  nw} pixels, using binomial
    weights within that window. Samples of {M} which are NAN or lie
    outside {M}'s domain are assumed to have zero weight. If all
    contributing samples for a sample of {R} have zero weight, the
    sample is set to NAN.
    
    The average is the weighted arithmetic mean if {harm} is false, or
    the weighted harmonic mean if {harm} is true.  
    
    A point {(x,y)} in the domain of {R} corresponds to point
    {(2*x,2*y)} of {A} if {nw} is even, and to {(2*x-0.5,2*y-0.5)} if
    {nw} is odd. */

r2_t float_image_mscale_point_shrink(r2_t *pA, int nw);
  /* If {R} is the result of {float_image_mscale_shrink(A,M,nw)}, 
    returns the point of {R}  domain that corresponds 
    to point {p} in {A}'s domain. */

r2_t float_image_mscale_point_expand(r2_t *pR, int nw);
  /* If {R} is the result of {float_image_mscale_shrink(A,M,nw)}, 
    returns the point of {A} domain that corresponds 
    to point {p} in {R}'s domain. */

/* !!! Add {float_image_mscale_expand} generalizing {pst_height_map_expand} !!! */

int float_image_mscale_rounding_bias(int n);
  /* A bias bit that may be useful to keep the image centered during reduction.
      For n =  0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 16 17 
          b =  0  1  0  0  0  1  0  1  0  1  0  0  0  1  0  0  0  1
     !!! Find a use for this procedure !!! 
  */

#endif
