/* See pst_height_map.h */
/* Last edited on 2010-03-04 19:05:23 by stolfi */

#include <assert.h>

#include <pst_height_map.h>
#include <float_image.h>

float_image_t *pst_height_map_expand(float_image_t *JZ, int NXI, int NYI)
  { 
    /* !!! use float_image_mscale !!! */
    assert(JZ->sz[0] == 1);
    int NXJ = JZ->sz[1]; assert(NXI/2 + 1 == NXJ);
    int NYJ = JZ->sz[2]; assert(NYI/2 + 1 == NYJ);
    float_image_t *IZ = float_image_new(1, NXI, NYI);
    int xI, yI;
    for(yI = 0; yI < NYI; yI++)
      { for(xI = 0; xI < NXI; xI++)
          { /* Get relevant samples from original image: */
            int xJ0 = xI/2, xJ1 = (xI+1)/2;
            int yJ0 = yI/2, yJ1 = (yI+1)/2;
            assert(xJ1 < NXJ); assert(yJ1 < NYJ);
            double d00 = float_image_get_sample(JZ, 0, xJ0, yJ0);
            double d01 = float_image_get_sample(JZ, 0, xJ0, yJ1);
            double d10 = float_image_get_sample(JZ, 0, xJ1, yJ0);
            double d11 = float_image_get_sample(JZ, 0, xJ1, yJ1);
            /* Average samples, and scale {Z} up by 2: */
            float v = (d00 + d01 + d10 + d11)/2;
            /* Store sample in expanded image: */
            float_image_set_sample(IZ, 0, xI, yI, v);
          }
      }
    return IZ;
  }

float_image_t *pst_height_map_compare(float_image_t *IZ, float_image_t *RZ, bool_t zero_mean)
  { assert(IZ->sz[0] == 1);
    assert(RZ->sz[0] == 1);
    int NX = IZ->sz[1]; assert(RZ->sz[1] == NX);
    int NY = IZ->sz[2]; assert(RZ->sz[2] == NY);
    float_image_t *EZ = float_image_new(1, NX, NY);
    int x, y;
    double sum = 0.0;
    for(y = 0; y < NY; y++)
      { for(x = 0; x < NX; x++)
          { /* Get relevant samples from original image: */
            double vI = float_image_get_sample(IZ, 0, x, y);
            double vR = float_image_get_sample(RZ, 0, x, y);
            /* Average samples, and scale {Z} up by 2: */
            double vE = vI - vR;
            sum += vE;
            /* Store sample in expanded image: */
            float_image_set_sample(EZ, 0, x, y, (float)vE);
          }
      }
    if (zero_mean)
      { double avg = sum/NX/NY;
        for(y = 0; y < NY; y++)
          { for(x = 0; x < NX; x++)
              { /* Get relevant samples from original image: */
                double vE = float_image_get_sample(EZ, 0, x, y);
                /* Store sample in expanded image: */
                float_image_set_sample(EZ, 0, x, y, (float)(vE - avg));
              }
          }
      }
    return EZ;
  }
