#ifndef pst_height_map_H
#define pst_height_map_H

/* pst_height_map.h -- procedures for working with height maps. */
/* Last edited on 2010-03-04 19:03:09 by stolfi */

#include <float_image.h>

/* HEIGHT MAPS
  
  A /height map/ is a single-channel float-valued image where
  the pixel value represents a height field {Z(X,Y)}. */

float_image_t *pst_height_map_expand(float_image_t *JZ, int NX, int NY);
  /* Given a height map {JZ}, returns another height map {IZ} with
    {NX} columns and {NY} rows.
    
    The original image must have size {NX/2 + 1} by {NY/2 + 1},
    rounded down. In the new image, pixels whose indices are both even
    are copied from the original, while pixels with one or two odd
    indices are interpolated. */

float_image_t *pst_height_map_compare(float_image_t *IZ, float_image_t *RZ, bool_t zero_mean);
  /* Returns a height map which is the difference {IZ-RZ} of the 
    two given height maps (which msut have the same size). 
    If {zero_mean} is TRUE, subtracts the mean values so that
    the mean difference is zero. */

#endif
