/* See pst_weight_map.h */
/* Last edited on 2010-03-06 13:57:35 by stolfi */

#define _GNU_SOURCE
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>

#include <float_image.h>

#include <pst_basic.h>
#include <pst_weight_map.h>

/* INTERNAL PROTOTYPES */
    
#define get_sample float_image_get_sample
#define set_sample float_image_set_sample
 
/* IMPLEMENTATIONS */

float_image_t *pst_weight_map_shrink(float_image_t *IW, bool_t harmonic)
  { int NC = IW->sz[0];
    int NXI = IW->sz[1]; int NXJ = (NXI+1)/2;
    int NYI = IW->sz[2]; int NYJ = (NYI+1)/2;
    float_image_t *JW = float_image_new(NC, NXJ, NYJ);
    int xJ, yJ, c;
    for(yJ = 0; yJ < NYJ; yJ++)
      { for(xJ = 0; xJ < NXJ; xJ++)
          { int xI = 2*xJ, yI = 2*yJ;
            for (c = 0; c < NC; c++)
              { /* Samples in original image (with border padding): */
                double w00 = get_sample(IW, c, xI, yI);
                double w01 = ((xI+1 < NXI) ? get_sample(IW, c, xI+1, yI) : w00);
                double w10 = ((yI+1 < NYI) ? get_sample(IW, c, xI, yI+1) : w00);
                double w11 = ((xI+1 < NXI) && (yI+1 < NYI) ? get_sample(IW, c, xI+1, yI+1) : w00);
                assert(w00 >= 0);
                assert(w01 >= 0);
                assert(w10 >= 0);
                assert(w11 >= 0);
                /* Average the weights: */
                float w;
                if (harmonic)
                  { w = 4/(1/w00 + 1/w01 + 1/w10 + 1/w11); /* Harmonic mean. */ }
                else
                  { w = (w00 + w01 + w10 + w11)/4; /* Arithmetic mean. */ }
                assert(w >= 0);
                assert(! isnan(w));
                 /* Set sample in shrunken image: */
                set_sample(JW, c, xJ, yJ, w);
              }
          }
      }
    return JW;
  }
