#define PROG_NAME "slope_to_height"
#define PROG_DESC "compute a heigth map from X and Y slope maps"
#define PROG_VERS "1.0"

#define slope_to_height_C_COPYRIGHT "Copyright  2005 by the State University of Campinas (UNICAMP)"

/* Last edited on 2010-03-04 19:05:48 by stolfi */

#define PROG_HELP \
  "  " PROG_NAME " \\\n" \
  "    [ -harmonicAvgW ] \\\n" \
  "    [ -debugZ ] [ -debugSys ] [ -debugDZ ] \\\n" \
  "    [ -debugPrefix {PREFIX} ] \\\n" \
  "    [ -compareZ {REF_Z_FNI_NAME} {ERR_Z_FNI_NAME} ] \\\n" \
  "    " argparser_help_info_HELP " \\\n" \
  "    {DZ_FNI_NAME} [ {W_FNI_NAME} ] \\\n" \
  "    > {Z_FNI_NAME}"

#define PROG_INFO \
  "NAME\n" \
  "  " PROG_NAME " - " PROG_DESC "\n" \
  "\n" \
  "SYNOPSIS\n" \
  PROG_HELP "\n" \
  "\n" \
  "DESCRIPTION\n" \
  PROG_INFO_DESC "\n" \
  "\n" \
  "OPTIONS\n" \
  PROG_INFO_OPTS "\n" \
  "\n" \
  argparser_help_info_HELP_INFO "\n" \
  "\n" \
  "SEE ALSO\n" \
  "  fni_to_pnm(1), pnm_to_fni(1).\n" \
  "\n" \
  "AUTHOR\n" \
  "  Created 2005-08-15 by Jorge Stolfi, UNICAMP.\n" \
  "MODIFICATION HISTORY\n" \
  "  2008-11-08 by J. Stolfi, IC-UNICAMP: added the weight map option.\n" \
  "\n" \
  "  2008-11-09 by J. Stolfi, IC-UNICAMP: added the reference height map.\n" \
  "\n" \
  "WARRANTY\n" \
  argparser_help_info_NO_WARRANTY "\n" \
  "\n" \
  "RIGHTS\n" \
  "  " slope_to_height_C_COPYRIGHT ".\n" \
  "\n" \
  argparser_help_info_STANDARD_RIGHTS
  
#define PROG_INFO_DESC \
  "  The program reads a /slope map/, consisting of" \
  " the {X} and {Y} derivatives of some terrain surface {Z(X,Y)}; and" \
  " computes the corresponding /height map/ {Z(X,Y)}.  Optionally, the" \
  " program may also compare the computed {Z} map with" \
  " a reference {Z} map provided by the user.\n" \
  "\n" \
  "  The input slope map is a two-channel float image, read from" \
  " the file called \"{DZ_FNI_NAME}\" which must be in FNI format" \
  " (see float_image.h).  Each pixel {DZ[X,Y]} of this image is" \
  " the gradient of the {Z} function, averaged" \
  " over the unit square cell (/pixel/) whose lower left corner" \
  " is the point {(X,Y)}.\n" \
  "\n" \
  "  If the {W_FNI_NAME} parameter is" \
  " given, the program also reads from that file a /weight map/, a" \
  " single-channel image that specifies the reliability" \
  " of the slope data.  Specifically, {W[X,Y]} is the reliability" \
  " for the slope datum {DZ[X,Y]}.  A zero weight means that the slope" \
  " datum should be ignored.\n" \
  "\n" \
  "  If either {DZ_FNI_NAME} or {W_FNI_NAME}" \
  " is \"-\", the corresponding map is read from standard input.\n" \
  "\n" \
  "  The output height map is a single-channel" \
  " image containing the {Z} values of the terrain most" \
  " compatible with the given slopes.  It includes an arbitrary" \
  " global integration constant for each maximal part of the" \
  " domain which is connected by paths with non-zero weight.  This" \
  " image has one more column and one more row than the input slope maps.  The" \
  " height map is written to standard output, also in" \
  " the FNI image format.\n" \
  "\n" \
  "  If the \"-compareZ {REF_Z_NAME} {ERR_Z_FNI_NAME}\" option is given, the program" \
  " also outputs an error map, defined as the difference between" \
  " the height map computed by the program and a reference map" \
  " read from the FNI-format file \"{REF_Z_NAME}\".  The reference" \
  " map must have the correct size for a height map,  namely one more column and" \
  " one more row than the input slope maps.  The error map is written" \
  " to the file \"{ERR_Z_FNI_NAME}\".\n" \
  "\n" \
  "  If the \"-debugZ\" option is given, the program" \
  " also writes out the intermediate (reduced-scale)" \
  "  solutions found, for debugging. The options" \
  " \"-debugDZ\" and \"-debugSys\" cause additional" \
  " debugging data to be written---namely, the slope" \
  " maps and the linear systems solved at each scale," \
  " respectively.  These debugging files are called" \
  " \"{PEFIX}-{LEVEL}-Z.fni\", \"{PEFIX}-{LEVEL}-dZ.fni\"," \
  " and \"{PEFIX}-{LEVEL}.sys\", respectively; where" \
  " {PEFIX} is the argument of the \"-debugPrefix\"" \
  " option (default \"debug\") and {LEVEL} is the" \
  " recursion depth."
  
#define PROG_INFO_OPTS \
  "  -compareZ {REF_Z_FNI_NAME} {ERR_Z_FNI_NAME}\n" \
  "    If this argument is present, the program" \
  " also outputs an error map, defined as the difference between" \
  " the height map computed by the program and a reference map" \
  " read from the FNI-format file \"{REF_Z_NAME}\".  The reference" \
  " map must have the correct size for a height map,  namely one more column and" \
  " one more row than the input slope maps.  The error map is written" \
  " to the file \"{ERR_Z_FNI_NAME}\".  If this option is" \
  " omitted, the reference map is not read and the error file is not written.\n" \
  "\n" \
  "  -debugZ\n" \
  "    Causes the program to write the solutions" \
  " (height maps) obtained at intermediate scales," \
  " in human-readable formats.\n" \
  "\n" \
  "  -debugDZ\n" \
  "    Causes the program to write the reduced versions" \
  " of the slope maps used at intermediate scales," \
  " in human-readable format.\n" \
  "\n" \
  "  -debugSys\n" \
  "    Causes the program to write the linear systems" \
  " used at intermediate scales.\n" \
  "\n" \
  "  -debugPrefix {PREFIX}\n" \
  "    Specifies the common prefix for debugging" \
  " file names (default \"debug\")."

#define _GNU_SOURCE
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include <values.h>
#include <stdint.h>
#include <math.h>

#include <bool.h>
#include <jsfile.h>
#include <argparser.h>
#include <float_image.h>

#include <pst_slope_map.h>
#include <pst_imgsys.h>
#include <pst_height_map.h>

/* COMMAND-LINE OPTIONS */

typedef struct options_t
  { bool_t harmonicAvgW;  /* Averaging for weight reduction: TRUE = harmonic, FALSE = arithmetic. */
    bool_t debugZ;        /* TRUE writes out the intermediate Z images. */
    bool_t debugDZ;       /* TRUE writes out the intermediate slope arrays. */
    bool_t debugSys;      /* TRUE writes the linear system(s). */
    char* debugPrefix;    /* Output name prefix for diagnostic files. */
    char* refZFileName;   /* File name of input reference height map (NULL if none). */
    char* errZFileName;   /* File name of output height error map (NULL if none). */
    char* slopeFileName;  /* File name of input slope file, or "-" for stdin. */
    char* weightFileName; /* File name of weight slope file, or "-" for stdin (NULL if none). */
  } options_t;

float_image_t *read_fni_file(char *fileName);
  /* Reads a FNI image from file "{fileName}" (which should include the extension ".fni").
    If {fileName} is "-", reads from standard input. */

void write_fni_file(float_image_t *I, char *fileName, int indent);
  /* Writes the float image {I} in a human-readable format, to a file
    called "{fileName}" (which should include the extension ".fni").
    If {fileName} is "-", writes the image to standard output.
    Diagnostic messages are indented by {indent}. */

options_t *parse_options(int argc, char **argv);
  /* Parses the command line arguments and packs them as an {options_t}. */

int main(int argc,char** argv);

/* IMPLEMENTATIONS */

int main(int argc, char** argv)
  {
    options_t *o = parse_options(argc, argv);
    
    fprintf(stderr, "Reading the slope map:\n");
    float_image_t *IDZ = read_fni_file(o->slopeFileName);
    assert(IDZ->sz[0] == 2); /* Must be a two-channel file. */
    
    float_image_t *IW;
    if (o->weightFileName == NULL)
      { IW = NULL; }
    else
      { fprintf(stderr, "Reading the weight image:\n");
        IW = read_fni_file(o->weightFileName);
        assert(IW->sz[0] == 1); /* Must be a single-channel file. */
      }
    
    fprintf(stderr, "Computing the height map:\n");
    float_image_t* OZ = pst_slope_map_to_depth_map_recursive
      ( IDZ, IW, 0, o->harmonicAvgW, o->debugDZ, o->debugZ, o->debugSys, o->debugPrefix );

    fprintf(stderr, "Writing the height map:\n");
    write_fni_file(OZ, "-", 0);
    
    if (o->refZFileName != NULL)
      { float_image_t *RZ = read_fni_file(o->refZFileName);
        float_image_t *EZ = pst_height_map_compare(OZ, RZ, TRUE);
        write_fni_file(EZ, o->errZFileName, 0);
      }

    fprintf(stderr, "Done!\n");
    return 0;
  }

float_image_t *read_fni_file(char *fileName)
  { demand(fileName != NULL, "file name not given");
    fprintf(stderr, "Reading %s ...", fileName);
    FILE *rd = open_read(fileName, FALSE);
    float_image_t *I = float_image_read(rd);
    if (rd != stdin) { fclose(rd); }
    fprintf(stderr, "\n");
    return I;
  }

void write_fni_file(float_image_t *I, char *fileName, int indent)
  { demand(fileName != NULL, "file name not given");
    fprintf(stderr, "%*sWriting %s ...", indent, "", fileName);
    FILE* wr = open_write(fileName, FALSE);
    float_image_write(wr, I);
    if (wr == stdout) { fflush(wr); } else { fclose(wr); }
    fprintf(stderr, "\n");
  }

options_t *parse_options(int argc, char **argv)
  { argparser_t *pp = argparser_new(stderr, argc, argv);
    argparser_set_help(pp, PROG_NAME " version " PROG_VERS ", usage:\n" PROG_HELP);
    argparser_set_info(pp, PROG_INFO);
    argparser_process_help_info_options(pp);
    
    options_t *o = (options_t *)notnull(malloc(sizeof(options_t)), "no mem"); 

    o->harmonicAvgW = argparser_keyword_present(pp, "-harmonicAvgW");
    
    if (argparser_keyword_present(pp, "-compareZ"))
      { o->refZFileName = argparser_get_next(pp); 
        o->errZFileName = argparser_get_next(pp);
      }
    else
      { o->refZFileName = NULL; o->errZFileName = NULL; }
    
    o->debugZ = argparser_keyword_present(pp, "-debugZ");
    o->debugDZ = argparser_keyword_present(pp, "-debugDZ");
    o->debugSys = argparser_keyword_present(pp, "-debugSys");
    if (argparser_keyword_present(pp, "-debugPrefix"))
      { o->debugPrefix = argparser_get_next(pp); }
    else
      { o->debugPrefix = "debug"; }

    argparser_skip_parsed(pp);
    
    o->slopeFileName = argparser_get_next_non_keyword(pp);
    if ((argparser_next(pp) == NULL) || (argparser_next_is_keyword(pp)))
      { o->weightFileName = NULL; }
    else
      { o->weightFileName = argparser_get_next_non_keyword(pp); }

    argparser_finish(pp);
    
    return o;
  }
